@@ -65,10 +65,12 @@ int main(int argc, char *argv[])
 	struct mddev_ident_s ident;
 	char *configfile = NULL;
 	char *cp;
+	char *update = NULL;
 	int scan = 0;
 	char devmode = 0;
 	int runstop = 0;
 	int readonly = 0;
+	int SparcAdjust = 0;
 	mddev_dev_t devlist = NULL;
 	mddev_dev_t *devlistend = & devlist;
 	mddev_dev_t dv;
@@ -80,6 +82,7 @@ int main(int argc, char *argv[])
 	char *mailaddr = NULL;
 	char *program = NULL;
 	int delay = 0;
+	int daemonise = 0;
 
 	int mdfd = -1;
 
@@ -267,7 +270,7 @@ int main(int argc, char *argv[])
 				exit(2);
 			}
 			if (sparedisks > 0 && level < 1 && level >= -1) {
-				fprintf(stderr, Name ": raid level %s is incompatible with spare-disks setting.\n",
+				fprintf(stderr, Name ": raid level %s is incompatible with spare-devices setting.\n",
 					optarg);
 				exit(2);
 			}
@@ -303,13 +306,13 @@ int main(int argc, char *argv[])
 		case O(CREATE,'n'):
 		case O(BUILD,'n'): /* number of raid disks */
 			if (raiddisks) {
-				fprintf(stderr, Name ": raid-disks set twice: %d and %s\n",
+				fprintf(stderr, Name ": raid-devices set twice: %d and %s\n",
 					raiddisks, optarg);
 				exit(2);
 			}
 			raiddisks = strtol(optarg, &c, 10);
 			if (!optarg[0] || *c || raiddisks<=0 || raiddisks > MD_SB_DISKS) {
-				fprintf(stderr, Name ": invalid number of raid disks: %s\n",
+				fprintf(stderr, Name ": invalid number of raid devices: %s\n",
 					optarg);
 				exit(2);
 			}
@@ -318,18 +321,18 @@ int main(int argc, char *argv[])
 
 		case O(CREATE,'x'): /* number of spare (eXtra) discs */
 			if (sparedisks) {
-				fprintf(stderr,Name ": spare-disks set twice: %d and %s\n",
+				fprintf(stderr,Name ": spare-devices set twice: %d and %s\n",
 					sparedisks, optarg);
 				exit(2);
 			}
 			if (level > -10 && level <= 0 && level >= -1) {
-				fprintf(stderr, Name ": spare-disks setting is incompatible with raid level %d\n",
+				fprintf(stderr, Name ": spare-devices setting is incompatible with raid level %d\n",
 					level);
 				exit(2);
 			}
 			sparedisks = strtol(optarg, &c, 10);
 			if (!optarg[0] || *c || sparedisks < 0 || sparedisks > MD_SB_DISKS - raiddisks) {
-				fprintf(stderr, Name ": invalid number of spare disks: %s\n",
+				fprintf(stderr, Name ": invalid number of spare-devices: %s\n",
 					optarg);
 				exit(2);
 			}
@@ -356,19 +359,37 @@ int main(int argc, char *argv[])
 			continue;
 
 		case O(ASSEMBLE,'m'): /* super-minor for array */
-			if (ident.super_minor >= 0) {
+			if (ident.super_minor != -1) {
 				fprintf(stderr, Name ": super-minor cannot be set twice.  "
 					"Second value: %s.\n", optarg);
 				exit(2);
 			}
-			ident.super_minor = strtoul(optarg, &cp, 10);
-			if (!optarg[0] || *cp) {
-				fprintf(stderr, Name ": Bad super-minor number: %s.\n", optarg);
-				exit(2);
+			if (strcmp(optarg, "dev")==0)
+				ident.super_minor = -2;
+			else {
+				ident.super_minor = strtoul(optarg, &cp, 10);
+				if (!optarg[0] || *cp) {
+					fprintf(stderr, Name ": Bad super-minor number: %s.\n", optarg);
+					exit(2);
+				}
 			}
 			continue;
 
+		case O(ASSEMBLE,'U'): /* update the superblock */
+			if (update) {
+				fprintf(stderr, Name ": Can only update one aspect of superblock, both %s and %s given.\n",
+					update, optarg);
+				exit(2);
+			}
+			update = optarg;
+			if (strcmp(update, "sparc2.2")==0) continue;
+			if (strcmp(update, "super-minor") == 0)
+				continue;
+			fprintf(stderr, Name ": '--update %s' invalid.  Only 'sparc2.2' or 'super-minor' supported\n",update);
+			exit(2);
+
 		case O(ASSEMBLE,'c'): /* config file */
+		case O(MISC, 'c'):
 		case O(MONITOR,'c'):
 			if (configfile) {
 				fprintf(stderr, Name ": configfile cannot be set twice.  "
@@ -413,6 +434,9 @@ int main(int argc, char *argv[])
 				}
 			}
 			continue;
+		case O(MONITOR,'f'): /* daemonise */
+			daemonise = 1;
+			continue;
 			
 
 			/* now the general management options.  Some are applicable
@@ -480,6 +504,13 @@ int main(int argc, char *argv[])
 			devmode = opt;
 			continue;
 
+		case O(MISC, 22):
+			if (devmode != 'E') {
+				fprintf(stderr, Name ": --sparc2.2 only allowed with --examine\n");
+				exit(2);
+			}
+			SparcAdjust = 1;
+			continue;
 		}
 		/* We have now processed all the valid options. Anything else is
 		 * an error
@@ -512,6 +543,11 @@ int main(int argc, char *argv[])
 		mdfd = open_mddev(devlist->devname);
 		if (mdfd < 0)
 			exit(1);
+		if (ident.super_minor == -2) {
+			struct stat stb;
+			fstat(mdfd, &stb);
+			ident.super_minor = MINOR(stb.st_rdev);
+		}
 	}
 
 	rv = 0;
@@ -529,11 +565,33 @@ int main(int argc, char *argv[])
 			rv = Manage_runstop(devlist->devname, mdfd, runstop);
 		break;
 	case ASSEMBLE:
-		if (!scan)
+		if (devs_found == 1 && ident.uuid_set == 0 &&
+		    ident.super_minor == -1 && !scan ) {
+			/* Only a device has been given, so get details from config file */
+			mddev_ident_t array_ident = conf_get_ident(configfile, devlist->devname);
+			mdfd = open_mddev(devlist->devname);
+			if (mdfd < 0)
+				rv |= 1;
+			else {
+				if (array_ident == NULL) {
+					fprintf(stderr, Name ": %s not identified in config file.\n",
+						devlist->devname);
+					rv |= 1;
+				}
+				else 
+					rv |= Assemble(devlist->devname, mdfd, array_ident, configfile,
+						       NULL,
+						       readonly, runstop, update, verbose, force);
+			}
+		} else if (!scan)
 			rv = Assemble(devlist->devname, mdfd, &ident, configfile,
 				      devlist->next,
-				      readonly, runstop, verbose, force);
-		else if (devs_found>0)
+				      readonly, runstop, update, verbose, force);
+		else if (devs_found>0) {
+			if (update && devs_found > 1) {
+				fprintf(stderr, Name ": can only update a single array at a time\n");
+				exit(1);
+			}
 			for (dv = devlist ; dv ; dv=dv->next) {
 				mddev_ident_t array_ident = conf_get_ident(configfile, dv->devname);
 				mdfd = open_mddev(dv->devname);
@@ -549,9 +607,9 @@ int main(int argc, char *argv[])
 				}
 				rv |= Assemble(dv->devname, mdfd, array_ident, configfile,
 					       NULL,
-					       readonly, runstop, verbose, force);
+					       readonly, runstop, update, verbose, force);
 			}
-		else {
+		} else {
 			mddev_ident_t array_list =  conf_get_ident(configfile, NULL);
 			if (!array_list) {
 				fprintf(stderr, Name ": No arrays found in config file\n");
@@ -570,7 +628,7 @@ int main(int argc, char *argv[])
 					rv |= Assemble(array_list->devname, mdfd,
 						       array_list, configfile,
 						       NULL,
-						       readonly, runstop, verbose, force);
+						       readonly, runstop, NULL, verbose, force);
 				}
 		}
 		break;
@@ -592,10 +650,10 @@ int main(int argc, char *argv[])
 			if (devlist == NULL)
 				devlist = conf_get_devs(configfile);
 			if (devlist == NULL) {
-				fprintf(stderr, Name ": No devices listed in %s\n", configfile);
+				fprintf(stderr, Name ": No devices listed in %s\n", configfile?configfile:DefaultConfFile);
 				exit(1);
 			}
-			rv = Examine(devlist, devlist?brief:!verbose, scan);
+			rv = Examine(devlist, scan?!verbose:brief, scan, SparcAdjust);
 		} else {
 			if (devlist == NULL) {
 				if ((devmode == 'S' ||devmode=='D') && scan) {
@@ -649,14 +707,13 @@ int main(int argc, char *argv[])
 		}
 		break;
 	case MONITOR:
-/*
 		if (!devlist && !scan) {
 			fprintf(stderr, Name ": Cannot monitor: need --scan or at least one device\n");
 			rv = 1;
 			break;
 		}
-*/		rv= Monitor(devlist, mailaddr, program,
-			    delay?delay:60, scan, configfile);
+		rv= Monitor(devlist, mailaddr, program,
+			    delay?delay:60, daemonise, scan, configfile);
 		break;
 	}
 	exit(rv);
